@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998-2000 Free Software Foundation, Inc.                   *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -97,7 +97,25 @@ extern "C" {
 /* location of user's personal info directory */
 #define PRIVATE_INFO	"%s/.terminfo"	/* plug getenv("HOME") into %s */
 
-#define DEBUG(n, a)	if (_nc_tracing & (1 << (n - 1))) _tracef a 
+/*
+ * Some traces are designed to be used via tic's verbose option (and similar in
+ * infocmp and toe) rather than the 'trace()' function.  So we use the bits
+ * above the normal trace() parameter as a debug-level.
+ */
+
+#define MAX_DEBUG_LEVEL 15
+#define DEBUG_LEVEL(n)	((n) << 12)	/* see TRACE_MAXIMUM */
+
+#define set_trace_level(n) \
+ 	_nc_tracing &= DEBUG_LEVEL(MAX_DEBUG_LEVEL), \
+	_nc_tracing |= DEBUG_LEVEL(n)
+
+#ifdef TRACE
+#define DEBUG(n, a)	if (_nc_tracing >= DEBUG_LEVEL(n)) _tracef a
+#else
+#define DEBUG(n, a)	/*nothing*/
+#endif
+
 extern unsigned _nc_tracing;
 extern void _nc_tracef(char *, ...) GCC_PRINTFLIKE(1,2);
 extern const char *_nc_visbuf(const char *);
@@ -133,6 +151,36 @@ struct token
 
 extern	struct token	_nc_curr_token;
 
+	/*
+	 * List of keynames with their corresponding code.
+	 */
+struct kn {
+	const char *name;
+	int code;
+};
+
+extern const struct kn _nc_key_names[];
+
+	/*
+	 * Offsets to string capabilities, with the corresponding functionkey
+	 * codes.
+	 */
+struct tinfo_fkeys {
+	unsigned offset;
+	chtype code;
+	};
+
+#if	BROKEN_LINKER
+
+#define	_nc_tinfo_fkeys	_nc_tinfo_fkeysf()
+extern struct tinfo_fkeys *_nc_tinfo_fkeysf(void);
+
+#else
+
+extern struct tinfo_fkeys _nc_tinfo_fkeys[];
+
+#endif
+
 	/*
 	 * The file comp_captab.c contains an array of these structures, one
 	 * per possible capability.  These are indexed by a hash table array of
@@ -161,6 +209,7 @@ extern const struct alias _nc_capalias_table[];
 extern const struct alias _nc_infoalias_table[];
 
 extern const struct name_table_entry	*_nc_get_table(bool);
+extern const struct name_table_entry	* const *_nc_get_hash_table(bool);
 
 #define NOTFOUND	((struct name_table_entry *) 0)
 
@@ -174,6 +223,8 @@ extern const struct name_table_entry	*_nc_get_table(bool);
 #define CANCELLED_NUMERIC	-2
 #define CANCELLED_STRING	(char *)-1
 
+#define VALID_BOOLEAN(s) ((s) >= 0)
+#define VALID_NUMERIC(s) ((s) >= 0)
 #define VALID_STRING(s) ((s) != CANCELLED_STRING && (s) != ABSENT_STRING)
 
 /* termcap entries longer than this may break old binaries */
@@ -217,10 +268,10 @@ extern void _nc_warning(const char *const,...) GCC_PRINTFLIKE(1,2);
 extern bool _nc_suppress_warnings;
 
 /* comp_expand.c: expand string into readable form */
-extern char *_nc_tic_expand(const char *, bool);
+extern char *_nc_tic_expand(const char *, bool, int);
 
 /* comp_scan.c: decode string from readable form */
-extern char _nc_trans_string(char *);
+extern char _nc_trans_string(char *, char *);
 
 /* captoinfo.c: capability conversion */
 extern char *_nc_captoinfo(const char *, const char *, int const);
