@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998,1999 Free Software Foundation, Inc.                   *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -39,96 +39,101 @@
 extern int malloc_errfd;	/* FIXME */
 #endif
 
-MODULE_ID("$Id: lib_freeall.c,v 1.14 1999/04/03 23:17:06 tom Exp $")
+MODULE_ID("$Id: lib_freeall.c,v 1.16 1999/11/28 01:34:11 tom Exp $")
 
-static void free_slk(SLK *p)
+static void
+free_slk(SLK *p)
 {
-	if (p != 0) {
-		FreeIfNeeded(p->ent);
-		FreeIfNeeded(p->buffer);
-		free(p);
-	}
+    if (p != 0) {
+	FreeIfNeeded(p->ent);
+	FreeIfNeeded(p->buffer);
+	free(p);
+    }
 }
 
-static void free_tries(struct tries *p)
+static void
+free_tries(struct tries *p)
 {
-	struct tries *q;
+    struct tries *q;
 
-	while (p != 0) {
-		q = p->sibling;
-		if (p->child != 0)
-			free_tries(p->child);
-		free(p);
-		p = q;
-	}
+    while (p != 0) {
+	q = p->sibling;
+	if (p->child != 0)
+	    free_tries(p->child);
+	free(p);
+	p = q;
+    }
 }
 
 /*
  * Free all ncurses data.  This is used for testing only (there's no practical
  * use for it as an extension).
  */
-void _nc_freeall(void)
+void
+_nc_freeall(void)
 {
-	WINDOWLIST *p, *q;
+    WINDOWLIST *p, *q;
 
 #if NO_LEAKS
-	_nc_free_tparm();
+    _nc_free_tparm();
 #endif
+    if (SP != 0) {
 	while (_nc_windows != 0) {
-		/* Delete only windows that're not a parent */
-		for (p = _nc_windows; p != 0; p = p->next) {
-			bool found = FALSE;
+	    /* Delete only windows that're not a parent */
+	    for (p = _nc_windows; p != 0; p = p->next) {
+		bool found = FALSE;
 
-			for (q = _nc_windows; q != 0; q = q->next) {
-				if ((p != q)
-				 && (q->win->_flags & _SUBWIN)
-				 && (p->win == q->win->_parent)) {
-					found = TRUE;
-					break;
-				}
-			}
-
-			if (!found) {
-				delwin(p->win);
-				break;
-			}
+		for (q = _nc_windows; q != 0; q = q->next) {
+		    if ((p != q)
+			&& (q->win->_flags & _SUBWIN)
+			&& (p->win == q->win->_parent)) {
+			found = TRUE;
+			break;
+		    }
 		}
+
+		if (!found) {
+		    delwin(p->win);
+		    break;
+		}
+	    }
 	}
 
-	if (SP != 0) {
-		free_tries (SP->_keytry);
-		free_tries (SP->_key_ok);
-	    	free_slk(SP->_slk);
-		FreeIfNeeded(SP->_color_pairs);
-		FreeIfNeeded(SP->_color_table);
-		/* it won't free buffer anyway */
-/*		_nc_set_buffer(SP->_ofp, FALSE);*/
+	free_tries(SP->_keytry);
+	free_tries(SP->_key_ok);
+	free_slk(SP->_slk);
+	FreeIfNeeded(SP->_color_pairs);
+	FreeIfNeeded(SP->_color_table);
 #if !BROKEN_LINKER
-		FreeAndNull(SP);
+	FreeAndNull(SP);
 #endif
-	}
-
-	if (cur_term != 0) {
-		_nc_free_termtype(&(cur_term->type));
-		free(cur_term);
-	}
+    }
 
+    if (cur_term != 0) {
+	_nc_free_termtype(&(cur_term->type));
+	free(cur_term);
+    }
 #ifdef TRACE
-	(void) _nc_trace_buf(-1, 0);
+    (void) _nc_trace_buf(-1, 0);
 #endif
 #if HAVE_LIBDBMALLOC
-	malloc_dump(malloc_errfd);
+    malloc_dump(malloc_errfd);
 #elif HAVE_LIBDMALLOC
 #elif HAVE_PURIFY
-	purify_all_inuse();
+    purify_all_inuse();
 #endif
 }
 
-void _nc_free_and_exit(int code)
+void
+_nc_free_and_exit(int code)
 {
-	_nc_freeall();
-	exit(code);
+    _nc_freeall();
+    exit(code);
 }
+
 #else
-void _nc_freeall(void) { }
+void
+_nc_freeall(void)
+{
+}
 #endif
