@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -31,7 +31,6 @@
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
  ****************************************************************************/
 
-
 /*
 **	lib_mvcur.c
 **
@@ -153,13 +152,13 @@
 #include <term.h>
 #include <ctype.h>
 
-MODULE_ID("$Id: lib_mvcur.c,v 1.57 1999/06/26 22:16:04 tom Exp $")
+MODULE_ID("$Id: lib_mvcur.c,v 1.67 2000/06/24 21:13:51 tom Exp $")
 
 #define STRLEN(s)       (s != 0) ? strlen(s) : 0
 
+#define CURRENT_ROW	SP->_cursrow	/* phys cursor row */
+#define CURRENT_COLUMN	SP->_curscol	/* phys cursor column */
 #define CURRENT_ATTR	SP->_current_attr	/* current phys attribute */
-#define CURRENT_ROW	SP->_cursrow		/* phys cursor row */
-#define CURRENT_COLUMN	SP->_curscol		/* phys cursor column */
 #define REAL_ATTR	SP->_current_attr	/* phys current attribute */
 #define WANT_CHAR(y, x)	SP->_newscr->_line[y].text[x]	/* desired state */
 #define BAUDRATE	cur_term->_baudrate	/* bits per second */
@@ -176,20 +175,21 @@ static float diff;
 static int normalized_cost(const char *const cap, int affcnt);
 
 #if !HAVE_STRSTR
-char * _nc_strstr(const char *haystack, const char *needle)
+char *
+_nc_strstr(const char *haystack, const char *needle)
 {
-	size_t len1 = strlen(haystack);
-	size_t len2 = strlen(needle);
-	char *result = 0;
+    size_t len1 = strlen(haystack);
+    size_t len2 = strlen(needle);
+    char *result = 0;
 
-	while ((len1 != 0) && (len1-- >= len2)) {
-		if (!strncmp(haystack, needle, len2)) {
-			result = haystack;
-			break;
-		}
-		haystack++;
+    while ((len1 != 0) && (len1-- >= len2)) {
+	if (!strncmp(haystack, needle, len2)) {
+	    result = haystack;
+	    break;
 	}
-	return result;
+	haystack++;
+    }
+    return result;
 }
 #endif
 
@@ -203,18 +203,20 @@ char * _nc_strstr(const char *haystack, const char *needle)
 static int
 trace_cost_of(const char *capname, const char *cap, int affcnt)
 {
-	int result = _nc_msec_cost(cap,affcnt);
-	TR(TRACE_CHARPUT|TRACE_MOVE, ("CostOf %s %d", capname, result));
-	return result;
+    int result = _nc_msec_cost(cap, affcnt);
+    TR(TRACE_CHARPUT | TRACE_MOVE,
+	("CostOf %s %d %s", capname, result, _nc_visbuf(cap)));
+    return result;
 }
 #define CostOf(cap,affcnt) trace_cost_of(#cap,cap,affcnt);
 
 static int
 trace_normalized_cost(const char *capname, const char *cap, int affcnt)
 {
-	int result = normalized_cost(cap,affcnt);
-	TR(TRACE_CHARPUT|TRACE_MOVE, ("NormalizedCost %s %d", capname, result));
-	return result;
+    int result = normalized_cost(cap, affcnt);
+    TR(TRACE_CHARPUT | TRACE_MOVE,
+	("NormalizedCost %s %d %s", capname, result, _nc_visbuf(cap)));
+    return result;
 }
 #define NormalizedCost(cap,affcnt) trace_normalized_cost(#cap,cap,affcnt);
 
@@ -225,68 +227,68 @@ trace_normalized_cost(const char *capname, const char *cap, int affcnt)
 
 #endif
 
-int _nc_msec_cost(const char *const cap, int affcnt)
+int
+_nc_msec_cost(const char *const cap, int affcnt)
 /* compute the cost of a given operation */
 {
     if (cap == 0)
-	return(INFINITY);
-    else
-    {
-	const	char	*cp;
-	float	cum_cost = 0;
+	return (INFINITY);
+    else {
+	const char *cp;
+	float cum_cost = 0;
 
-	for (cp = cap; *cp; cp++)
-	{
+	for (cp = cap; *cp; cp++) {
 	    /* extract padding, either mandatory or required */
-	    if (cp[0] == '$' && cp[1] == '<' && strchr(cp, '>'))
-	    {
-		float	number = 0;
+	    if (cp[0] == '$' && cp[1] == '<' && strchr(cp, '>')) {
+		float number = 0;
 
-		for (cp += 2; *cp != '>'; cp++)
-		{
+		for (cp += 2; *cp != '>'; cp++) {
 		    if (isdigit(*cp))
 			number = number * 10 + (*cp - '0');
-		    else if (*cp == '.')
-			number += (*++cp - 10) / 10.0;
 		    else if (*cp == '*')
 			number *= affcnt;
+		    else if (*cp == '.' && (*++cp != '>') && isdigit(*cp))
+			number += (*cp - '0') / 10.0;
 		}
 
-		cum_cost += number * 10;
-	    }
-	    else
+#ifdef NCURSES_NO_PADDING
+		if (!(SP->_no_padding))
+#endif
+		    cum_cost += number * 10;
+	    } else
 		cum_cost += SP->_char_padding;
 	}
 
-	return((int)cum_cost);
+	return ((int) cum_cost);
     }
 }
 
-static int normalized_cost(const char *const cap, int affcnt)
+static int
+normalized_cost(const char *const cap, int affcnt)
 /* compute the effective character-count for an operation (round up) */
 {
-	int cost = _nc_msec_cost(cap, affcnt);
-	if (cost != INFINITY)
-		cost = (cost + SP->_char_padding - 1) / SP->_char_padding;
-	return cost;
+    int cost = _nc_msec_cost(cap, affcnt);
+    if (cost != INFINITY)
+	cost = (cost + SP->_char_padding - 1) / SP->_char_padding;
+    return cost;
 }
 
-static void reset_scroll_region(void)
+static void
+reset_scroll_region(void)
 /* Set the scroll-region to a known state (the default) */
 {
-    if (change_scroll_region)
-    {
+    if (change_scroll_region) {
 	TPUTS_TRACE("change_scroll_region");
 	putp(tparm(change_scroll_region, 0, screen_lines - 1));
     }
 }
 
-void _nc_mvcur_resume(void)
+void
+_nc_mvcur_resume(void)
 /* what to do at initialization time and after each shellout */
 {
     /* initialize screen for cursor access */
-    if (enter_ca_mode)
-    {
+    if (enter_ca_mode) {
 	TPUTS_TRACE("enter_ca_mode");
 	putp(enter_ca_mode);
     }
@@ -304,15 +306,15 @@ void _nc_mvcur_resume(void)
     SP->_cursrow = SP->_curscol = -1;
 
     /* restore cursor shape */
-    if (SP->_cursor != -1)
-    {
+    if (SP->_cursor != -1) {
 	int cursor = SP->_cursor;
 	SP->_cursor = -1;
-	curs_set (cursor);
+	curs_set(cursor);
     }
 }
 
-void _nc_mvcur_init(void)
+void
+_nc_mvcur_init(void)
 /* initialize the cost structure */
 {
     /*
@@ -321,15 +323,15 @@ void _nc_mvcur_init(void)
     SP->_char_padding = (9 * 1000 * 10) / (BAUDRATE > 0 ? BAUDRATE : 9600);
     if (SP->_char_padding <= 0)
 	SP->_char_padding = 1;	/* must be nonzero */
-    TR(TRACE_CHARPUT|TRACE_MOVE, ("char_padding %d msecs", SP->_char_padding));
+    TR(TRACE_CHARPUT | TRACE_MOVE, ("char_padding %d msecs", SP->_char_padding));
 
     /* non-parameterized local-motion strings */
-    SP->_cr_cost   = CostOf(carriage_return, 0);
+    SP->_cr_cost = CostOf(carriage_return, 0);
     SP->_home_cost = CostOf(cursor_home, 0);
-    SP->_ll_cost   = CostOf(cursor_to_ll, 0);
+    SP->_ll_cost = CostOf(cursor_to_ll, 0);
 #if USE_HARD_TABS
-    SP->_ht_cost   = CostOf(tab, 0);
-    SP->_cbt_cost  = CostOf(back_tab, 0);
+    SP->_ht_cost = CostOf(tab, 0);
+    SP->_cbt_cost = CostOf(back_tab, 0);
 #endif /* USE_HARD_TABS */
     SP->_cub1_cost = CostOf(cursor_left, 0);
     SP->_cuf1_cost = CostOf(cursor_right, 0);
@@ -357,7 +359,7 @@ void _nc_mvcur_init(void)
      *
      * (1) They never have * padding.  In the entire master terminfo database
      *     as of March 1995, only the obsolete Zenith Z-100 pc violates this.
-     *	   (Proportional padding is found mainly in insert, delete and scroll
+     *     (Proportional padding is found mainly in insert, delete and scroll
      *     capabilities).
      *
      * (2) The average case of cup has two two-digit parameters.  Strictly,
@@ -375,34 +377,38 @@ void _nc_mvcur_init(void)
      * All these averages depend on the assumption that all parameter values
      * are equally probable.
      */
-    SP->_cup_cost  = CostOf(tparm(SP->_address_cursor, 23, 23), 1);
-    SP->_cub_cost  = CostOf(tparm(parm_left_cursor, 23), 1);
-    SP->_cuf_cost  = CostOf(tparm(parm_right_cursor, 23), 1);
-    SP->_cud_cost  = CostOf(tparm(parm_down_cursor, 23), 1);
-    SP->_cuu_cost  = CostOf(tparm(parm_up_cursor, 23), 1);
-    SP->_hpa_cost  = CostOf(tparm(column_address, 23), 1);
-    SP->_vpa_cost  = CostOf(tparm(row_address, 23), 1);
+    SP->_cup_cost = CostOf(tparm(SP->_address_cursor, 23, 23), 1);
+    SP->_cub_cost = CostOf(tparm(parm_left_cursor, 23), 1);
+    SP->_cuf_cost = CostOf(tparm(parm_right_cursor, 23), 1);
+    SP->_cud_cost = CostOf(tparm(parm_down_cursor, 23), 1);
+    SP->_cuu_cost = CostOf(tparm(parm_up_cursor, 23), 1);
+    SP->_hpa_cost = CostOf(tparm(column_address, 23), 1);
+    SP->_vpa_cost = CostOf(tparm(row_address, 23), 1);
 
     /* non-parameterized screen-update strings */
-    SP->_ed_cost   = NormalizedCost(clr_eos, 1);
-    SP->_el_cost   = NormalizedCost(clr_eol, 1);
-    SP->_el1_cost  = NormalizedCost(clr_bol, 1);
+    SP->_ed_cost = NormalizedCost(clr_eos, 1);
+    SP->_el_cost = NormalizedCost(clr_eol, 1);
+    SP->_el1_cost = NormalizedCost(clr_bol, 1);
     SP->_dch1_cost = NormalizedCost(delete_character, 1);
     SP->_ich1_cost = NormalizedCost(insert_character, 1);
 
     /* parameterized screen-update strings */
-    SP->_dch_cost  = NormalizedCost(tparm(parm_dch, 23), 1);
-    SP->_ich_cost  = NormalizedCost(tparm(parm_ich, 23), 1);
-    SP->_ech_cost  = NormalizedCost(tparm(erase_chars, 23), 1);
-    SP->_rep_cost  = NormalizedCost(tparm(repeat_char, ' ', 23), 1);
+    SP->_dch_cost = NormalizedCost(tparm(parm_dch, 23), 1);
+    SP->_ich_cost = NormalizedCost(tparm(parm_ich, 23), 1);
+    SP->_ech_cost = NormalizedCost(tparm(erase_chars, 23), 1);
+    SP->_rep_cost = NormalizedCost(tparm(repeat_char, ' ', 23), 1);
 
     SP->_cup_ch_cost = NormalizedCost(tparm(SP->_address_cursor, 23, 23), 1);
     SP->_hpa_ch_cost = NormalizedCost(tparm(column_address, 23), 1);
+    SP->_cuf_ch_cost = NormalizedCost(tparm(parm_right_cursor, 23), 1);
+    SP->_inline_cost = min(SP->_cup_ch_cost,
+	min(SP->_hpa_ch_cost,
+	    SP->_cuf_ch_cost));
 
     /* pre-compute some capability lengths */
     SP->_carriage_return_length = STRLEN(carriage_return);
-    SP->_cursor_home_length     = STRLEN(cursor_home);
-    SP->_cursor_to_ll_length    = STRLEN(cursor_to_ll);
+    SP->_cursor_home_length = STRLEN(cursor_home);
+    SP->_cursor_to_ll_length = STRLEN(cursor_to_ll);
 
     /*
      * If save_cursor is used within enter_ca_mode, we should not use it for
@@ -411,8 +417,8 @@ void _nc_mvcur_init(void)
      * feature.
      */
     if (save_cursor != 0
-     && enter_ca_mode != 0
-     && strstr(enter_ca_mode, save_cursor) != 0) {
+	&& enter_ca_mode != 0
+	&& strstr(enter_ca_mode, save_cursor) != 0) {
 	T(("...suppressed sc/rc capability due to conflict with smcup/rmcup"));
 	save_cursor = 0;
 	restore_cursor = 0;
@@ -426,7 +432,8 @@ void _nc_mvcur_init(void)
     _nc_mvcur_resume();
 }
 
-void _nc_mvcur_wrap(void)
+void
+_nc_mvcur_wrap(void)
 /* wrap up cursor-addressing mode */
 {
     /* leave cursor at screen bottom */
@@ -436,8 +443,7 @@ void _nc_mvcur_wrap(void)
     if (SP->_cursor != -1)
 	curs_set(1);
 
-    if (exit_ca_mode)
-    {
+    if (exit_ca_mode) {
 	TPUTS_TRACE("exit_ca_mode");
 	putp(exit_ca_mode);
     }
@@ -462,24 +468,24 @@ void _nc_mvcur_wrap(void)
  * Perform repeated-append, returning cost
  */
 static inline int
-repeated_append (int total, int num, int repeat, char *dst, const char *src)
+repeated_append(int total, int num, int repeat, char *dst, const char *src)
 {
-	register size_t src_len = strlen(src);
-	register size_t dst_len = STRLEN(dst);
+    register size_t src_len = strlen(src);
+    register size_t dst_len = STRLEN(dst);
 
-	if ((dst_len + repeat * src_len) < OPT_SIZE-1) {
-		total += (num * repeat);
-		if (dst) {
-		    dst += dst_len;
-		    while (repeat-- > 0) {
-			(void) strcpy(dst, src);
-			dst += src_len;
-		    }
-		}
-	} else {
-		total = INFINITY;
+    if ((dst_len + repeat * src_len) < OPT_SIZE - 1) {
+	total += (num * repeat);
+	if (dst) {
+	    dst += dst_len;
+	    while (repeat-- > 0) {
+		(void) strcpy(dst, src);
+		dst += src_len;
+	    }
 	}
-	return total;
+    } else {
+	total = INFINITY;
+    }
+    return total;
 }
 
 #ifndef NO_OPTIMIZE
@@ -494,56 +500,47 @@ repeated_append (int total, int num, int repeat, char *dst, const char *src)
 /* Note: we'd like to inline this for speed, but GNU C barfs on the attempt. */
 
 static int
-relative_move(char *result, int from_y,int from_x,int to_y,int to_x, bool ovw)
+relative_move(char *result, int from_y, int from_x, int to_y, int to_x, bool ovw)
 /* move via local motions (cuu/cuu1/cud/cud1/cub1/cub/cuf1/cuf/vpa/hpa) */
 {
-    int		n, vcost = 0, hcost = 0;
+    int n, vcost = 0, hcost = 0;
 
     if (result)
 	result[0] = '\0';
 
-    if (to_y != from_y)
-    {
+    if (to_y != from_y) {
 	vcost = INFINITY;
 
-	if (row_address)
-	{
+	if (row_address) {
 	    if (result)
 		(void) strcpy(result, tparm(row_address, to_y));
 	    vcost = SP->_vpa_cost;
 	}
 
-	if (to_y > from_y)
-	{
+	if (to_y > from_y) {
 	    n = (to_y - from_y);
 
-	    if (parm_down_cursor && SP->_cud_cost < vcost)
-	    {
+	    if (parm_down_cursor && SP->_cud_cost < vcost) {
 		if (result)
 		    (void) strcpy(result, tparm(parm_down_cursor, n));
 		vcost = SP->_cud_cost;
 	    }
 
-	    if (cursor_down && (n * SP->_cud1_cost < vcost))
-	    {
+	    if (cursor_down && (n * SP->_cud1_cost < vcost)) {
 		if (result)
 		    result[0] = '\0';
 		vcost = repeated_append(0, SP->_cud1_cost, n, result, cursor_down);
 	    }
-	}
-	else /* (to_y < from_y) */
-	{
+	} else {		/* (to_y < from_y) */
 	    n = (from_y - to_y);
 
-	    if (parm_up_cursor && SP->_cup_cost < vcost)
-	    {
+	    if (parm_up_cursor && SP->_cup_cost < vcost) {
 		if (result)
 		    (void) strcpy(result, tparm(parm_up_cursor, n));
 		vcost = SP->_cup_cost;
 	    }
 
-	    if (cursor_up && (n * SP->_cuu1_cost < vcost))
-	    {
+	    if (cursor_up && (n * SP->_cuu1_cost < vcost)) {
 		if (result)
 		    result[0] = '\0';
 		vcost = repeated_append(0, SP->_cuu1_cost, n, result, cursor_up);
@@ -551,53 +548,47 @@ relative_move(char *result, int from_y,int from_x,int to_y,int to_x, bool ovw)
 	}
 
 	if (vcost == INFINITY)
-	    return(INFINITY);
+	    return (INFINITY);
     }
 
     if (result)
 	result += strlen(result);
 
-    if (to_x != from_x)
-    {
-	char	str[OPT_SIZE];
+    if (to_x != from_x) {
+	char str[OPT_SIZE];
 
 	hcost = INFINITY;
 
-	if (column_address)
-	{
+	if (column_address) {
 	    if (result)
 		(void) strcpy(result, tparm(column_address, to_x));
 	    hcost = SP->_hpa_cost;
 	}
 
-	if (to_x > from_x)
-	{
+	if (to_x > from_x) {
 	    n = to_x - from_x;
 
-	    if (parm_right_cursor && SP->_cuf_cost < hcost)
-	    {
+	    if (parm_right_cursor && SP->_cuf_cost < hcost) {
 		if (result)
 		    (void) strcpy(result, tparm(parm_right_cursor, n));
 		hcost = SP->_cuf_cost;
 	    }
 
-	    if (cursor_right)
-	    {
-		int	lhcost = 0;
+	    if (cursor_right) {
+		int lhcost = 0;
 
 		str[0] = '\0';
 
 #if USE_HARD_TABS
 		/* use hard tabs, if we have them, to do as much as possible */
-		if (init_tabs > 0 && tab)
-		{
-		    int	nxt, fr;
+		if (init_tabs > 0 && tab) {
+		    int nxt, fr;
 
-		    for (fr = from_x; (nxt = NEXTTAB(fr)) <= to_x; fr = nxt)
-		    {
-			lhcost = repeated_append(lhcost, SP->_ht_cost, 1, str, tab);
+		    for (fr = from_x; (nxt = NEXTTAB(fr)) <= to_x; fr = nxt) {
+			lhcost = repeated_append(lhcost, SP->_ht_cost, 1,
+			    str, tab);
 			if (lhcost == INFINITY)
-				break;
+			    break;
 		    }
 
 		    n = to_x - fr;
@@ -614,11 +605,11 @@ relative_move(char *result, int from_y,int from_x,int to_y,int to_x, bool ovw)
 		 * screen.
 		 */
 		if (ovw
-		 && n > 0
-		 && vcost == 0
-		 && str[0] == '\0'
-		 && isdigit(TextOf(WANT_CHAR(to_y, from_x))))
-			ovw = FALSE;
+		    && n > 0
+		    && vcost == 0
+		    && str[0] == '\0'
+		    && isdigit(TextOf(WANT_CHAR(to_y, from_x))))
+		    ovw = FALSE;
 #endif
 		/*
 		 * If we have no attribute changes, overwrite is cheaper.
@@ -628,21 +619,18 @@ relative_move(char *result, int from_y,int from_x,int to_y,int to_x, bool ovw)
 		 * and the time the structure WANT_CHAR would access has been
 		 * updated.
 		 */
-		if (ovw)
-		{
-		    int	i;
+		if (ovw) {
+		    int i;
 
 		    for (i = 0; i < n; i++)
-			if ((WANT_CHAR(to_y, from_x + i) & A_ATTRIBUTES) != CURRENT_ATTR)
-			{
+			if ((WANT_CHAR(to_y, from_x + i) & A_ATTRIBUTES) != CURRENT_ATTR) {
 			    ovw = FALSE;
 			    break;
 			}
 		}
-		if (ovw)
-		{
-		    char	*sp;
-		    int	i;
+		if (ovw) {
+		    char *sp;
+		    int i;
 
 		    sp = str + strlen(str);
 
@@ -650,48 +638,41 @@ relative_move(char *result, int from_y,int from_x,int to_y,int to_x, bool ovw)
 			*sp++ = WANT_CHAR(to_y, from_x + i);
 		    *sp = '\0';
 		    lhcost += n * SP->_char_padding;
-		}
-		else
+		} else
 #endif /* defined(REAL_ATTR) && defined(WANT_CHAR) */
 		{
 		    lhcost = repeated_append(lhcost, SP->_cuf1_cost, n, str, cursor_right);
 		}
 
-		if (lhcost < hcost)
-		{
+		if (lhcost < hcost) {
 		    if (result)
 			(void) strcpy(result, str);
 		    hcost = lhcost;
 		}
 	    }
-	}
-	else /* (to_x < from_x) */
-	{
+	} else {		/* (to_x < from_x) */
 	    n = from_x - to_x;
 
-	    if (parm_left_cursor && SP->_cub_cost < hcost)
-	    {
+	    if (parm_left_cursor && SP->_cub_cost < hcost) {
 		if (result)
 		    (void) strcpy(result, tparm(parm_left_cursor, n));
 		hcost = SP->_cub_cost;
 	    }
 
-	    if (cursor_left)
-	    {
-		int	lhcost = 0;
+	    if (cursor_left) {
+		int lhcost = 0;
 
 		str[0] = '\0';
 
 #if USE_HARD_TABS
-		if (init_tabs > 0 && back_tab)
-		{
-		    int	nxt, fr;
+		if (init_tabs > 0 && back_tab) {
+		    int nxt, fr;
 
-		    for (fr = from_x; (nxt = LASTTAB(fr)) >= to_x; fr = nxt)
-		    {
-			lhcost = repeated_append(lhcost, SP->_cbt_cost, 1, str, back_tab);
+		    for (fr = from_x; (nxt = LASTTAB(fr)) >= to_x; fr = nxt) {
+			lhcost = repeated_append(lhcost, SP->_cbt_cost, 1,
+			    str, back_tab);
 			if (lhcost == INFINITY)
-				break;
+			    break;
 		    }
 
 		    n = fr - to_x;
@@ -700,8 +681,7 @@ relative_move(char *result, int from_y,int from_x,int to_y,int to_x, bool ovw)
 
 		lhcost = repeated_append(lhcost, SP->_cub1_cost, n, str, cursor_left);
 
-		if (lhcost < hcost)
-		{
+		if (lhcost < hcost) {
 		    if (result)
 			(void) strcpy(result, str);
 		    hcost = lhcost;
@@ -710,10 +690,10 @@ relative_move(char *result, int from_y,int from_x,int to_y,int to_x, bool ovw)
 	}
 
 	if (hcost == INFINITY)
-	    return(INFINITY);
+	    return (INFINITY);
     }
 
-    return(vcost + hcost);
+    return (vcost + hcost);
 }
 #endif /* !NO_OPTIMIZE */
 
@@ -729,11 +709,12 @@ relative_move(char *result, int from_y,int from_x,int to_y,int to_x, bool ovw)
  */
 
 static inline int
-onscreen_mvcur(int yold,int xold,int ynew,int xnew, bool ovw)
+onscreen_mvcur(int yold, int xold, int ynew, int xnew, bool ovw)
 /* onscreen move from (yold, xold) to (ynew, xnew) */
 {
-    char	use[OPT_SIZE], *sp;
-    int		tactic = 0, newcost, usecost = INFINITY;
+    char use[OPT_SIZE], *sp;
+    int tactic = 0, newcost, usecost = INFINITY;
+    int t5_cr_cost;
 
 #if defined(MAIN) || defined(NCURSES_TEST)
     struct timeval before, after;
@@ -743,8 +724,7 @@ onscreen_mvcur(int yold,int xold,int ynew,int xnew, bool ovw)
 
     /* tactic #0: use direct cursor addressing */
     sp = tparm(SP->_address_cursor, ynew, xnew);
-    if (sp)
-    {
+    if (sp) {
 	tactic = 0;
 	(void) strcpy(use, sp);
 	usecost = SP->_cup_cost;
@@ -762,11 +742,9 @@ onscreen_mvcur(int yold,int xold,int ynew,int xnew, bool ovw)
 	 * (like, say, local-movement \n getting mapped to some obscure
 	 * character because A_ALTCHARSET is on).
 	 */
-	if (yold == -1 || xold == -1 || NOT_LOCAL(yold, xold, ynew, xnew))
-	{
+	if (yold == -1 || xold == -1 || NOT_LOCAL(yold, xold, ynew, xnew)) {
 #if defined(MAIN) || defined(NCURSES_TEST)
-	    if (!profiling)
-	    {
+	    if (!profiling) {
 		(void) fputs("nonlocal\n", stderr);
 		goto nonlocal;	/* always run the optimizer if profiling */
 	    }
@@ -775,40 +753,36 @@ onscreen_mvcur(int yold,int xold,int ynew,int xnew, bool ovw)
 #endif /* MAIN */
 	}
     }
-
 #ifndef NO_OPTIMIZE
     /* tactic #1: use local movement */
     if (yold != -1 && xold != -1
-		&& ((newcost=relative_move(NULL, yold, xold, ynew, xnew, ovw))!=INFINITY)
-		&& newcost < usecost)
-    {
+	&& ((newcost = relative_move(NULL, yold, xold, ynew, xnew, ovw)) != INFINITY)
+	&& newcost < usecost) {
 	tactic = 1;
 	usecost = newcost;
     }
 
     /* tactic #2: use carriage-return + local movement */
     if (yold != -1 && carriage_return
-		&& ((newcost=relative_move(NULL, yold,0,ynew,xnew, ovw)) != INFINITY)
-		&& SP->_cr_cost + newcost < usecost)
-    {
+	&& ((newcost = relative_move(NULL, yold, 0, ynew, xnew, ovw)) != INFINITY)
+	&& SP->_cr_cost + newcost < usecost) {
 	tactic = 2;
 	usecost = SP->_cr_cost + newcost;
     }
 
     /* tactic #3: use home-cursor + local movement */
     if (cursor_home
-	&& ((newcost=relative_move(NULL, 0, 0, ynew, xnew, ovw)) != INFINITY)
-	&& SP->_home_cost + newcost < usecost)
-    {
+	&& ((newcost = relative_move(NULL, 0, 0, ynew, xnew, ovw)) != INFINITY)
+	&& SP->_home_cost + newcost < usecost) {
 	tactic = 3;
 	usecost = SP->_home_cost + newcost;
     }
 
     /* tactic #4: use home-down + local movement */
     if (cursor_to_ll
-	&& ((newcost=relative_move(NULL, screen_lines-1, 0, ynew, xnew, ovw)) != INFINITY)
-	&& SP->_ll_cost + newcost < usecost)
-    {
+	&& ((newcost = relative_move(NULL, screen_lines - 1, 0, ynew, xnew,
+		    ovw)) != INFINITY)
+	&& SP->_ll_cost + newcost < usecost) {
 	tactic = 4;
 	usecost = SP->_ll_cost + newcost;
     }
@@ -817,49 +791,46 @@ onscreen_mvcur(int yold,int xold,int ynew,int xnew, bool ovw)
      * tactic #5: use left margin for wrap to right-hand side,
      * unless strange wrap behavior indicated by xenl might hose us.
      */
+    t5_cr_cost = (xold > 0 ? SP->_cr_cost : 0);
     if (auto_left_margin && !eat_newline_glitch
 	&& yold > 0 && cursor_left
-	&& ((newcost=relative_move(NULL, yold-1, screen_columns-1, ynew, xnew, ovw)) != INFINITY)
-	&& SP->_cr_cost + SP->_cub1_cost + newcost + newcost < usecost)
-    {
+	&& ((newcost = relative_move(NULL, yold - 1, screen_columns - 1,
+		    ynew, xnew, ovw)) != INFINITY)
+	&& t5_cr_cost + SP->_cub1_cost + newcost < usecost) {
 	tactic = 5;
-	usecost = SP->_cr_cost + SP->_cub1_cost + newcost;
+	usecost = t5_cr_cost + SP->_cub1_cost + newcost;
     }
 
     /*
      * These cases are ordered by estimated relative frequency.
      */
-    if (tactic)
-    {
-	if (tactic == 1)
-	    (void) relative_move(use, yold, xold, ynew, xnew, ovw);
-	else if (tactic == 2)
-	{
-	    (void) strcpy(use, carriage_return);
-	    (void) relative_move(use + SP->_carriage_return_length,
-				 yold,0,ynew,xnew, ovw);
-	}
-	else if (tactic == 3)
-	{
-	    (void) strcpy(use, cursor_home);
-	    (void) relative_move(use + SP->_cursor_home_length,
-				 0, 0, ynew, xnew, ovw);
-	}
-	else if (tactic == 4)
-	{
-	    (void) strcpy(use, cursor_to_ll);
-	    (void) relative_move(use + SP->_cursor_to_ll_length,
-				 screen_lines-1, 0, ynew, xnew, ovw);
-	}
-	else /* if (tactic == 5) */
-	{
-	    use[0] = '\0';
-	    if (xold > 0)
-		(void) strcat(use, carriage_return);
-	    (void) strcat(use, cursor_left);
-	    (void) relative_move(use + strlen(use),
-				 yold-1, screen_columns-1, ynew, xnew, ovw);
-	}
+    switch (tactic) {
+    case 1:
+	(void) relative_move(use, yold, xold, ynew, xnew, ovw);
+	break;
+    case 2:
+	(void) strcpy(use, carriage_return);
+	(void) relative_move(use + SP->_carriage_return_length,
+	    yold, 0, ynew, xnew, ovw);
+	break;
+    case 3:
+	(void) strcpy(use, cursor_home);
+	(void) relative_move(use + SP->_cursor_home_length,
+	    0, 0, ynew, xnew, ovw);
+	break;
+    case 4:
+	(void) strcpy(use, cursor_to_ll);
+	(void) relative_move(use + SP->_cursor_to_ll_length,
+	    screen_lines - 1, 0, ynew, xnew, ovw);
+	break;
+    case 5:
+	use[0] = '\0';
+	if (xold > 0)
+	    (void) strcat(use, carriage_return);
+	(void) strcat(use, cursor_left);
+	(void) relative_move(use + strlen(use),
+	    yold - 1, screen_columns - 1, ynew, xnew, ovw);
+	break;
     }
 #endif /* !NO_OPTIMIZE */
 
@@ -868,28 +839,28 @@ onscreen_mvcur(int yold,int xold,int ynew,int xnew, bool ovw)
     diff = after.tv_usec - before.tv_usec
 	+ (after.tv_sec - before.tv_sec) * 1000000;
     if (!profiling)
-	(void) fprintf(stderr, "onscreen: %d msec, %f 28.8Kbps char-equivalents\n",
-		       (int)diff, diff/288);
+	(void) fprintf(stderr,
+	    "onscreen: %d msec, %f 28.8Kbps char-equivalents\n",
+	    (int) diff, diff / 288);
 #endif /* MAIN */
 
- nonlocal:
-    if (usecost != INFINITY)
-    {
+  nonlocal:
+    if (usecost != INFINITY) {
 	TPUTS_TRACE("mvcur");
 	tputs(use, 1, _nc_outch);
-	return(OK);
-    }
-    else
-	return(ERR);
+	return (OK);
+    } else
+	return (ERR);
 }
 
-int mvcur(int yold, int xold, int ynew, int xnew)
+int
+mvcur(int yold, int xold, int ynew, int xnew)
 /* optimized cursor move from (yold, xold) to (ynew, xnew) */
 {
     TR(TRACE_MOVE, ("mvcur(%d,%d,%d,%d) called", yold, xold, ynew, xnew));
 
     if (yold == ynew && xold == xnew)
-	return(OK);
+	return (OK);
 
     /*
      * Most work here is rounding for terminal boundaries getting the
@@ -897,40 +868,33 @@ int mvcur(int yold, int xold, int ynew, int xnew)
      * rolling up the screen to get ynew on the screen.
      */
 
-    if (xnew >= screen_columns)
-    {
+    if (xnew >= screen_columns) {
 	ynew += xnew / screen_columns;
 	xnew %= screen_columns;
     }
-    if (xold >= screen_columns)
-    {
-	int	l;
+    if (xold >= screen_columns) {
+	int l;
 
 	l = (xold + 1) / screen_columns;
 	yold += l;
 	if (yold >= screen_lines)
-		l -= (yold - screen_lines - 1);
+	    l -= (yold - screen_lines - 1);
 
 	while (l > 0) {
-		if (newline)
-		{
-			TPUTS_TRACE("newline");
-			tputs(newline, 0, _nc_outch);
-		}
-		else
-			putchar('\n');
-		l--;
-		if (xold > 0)
-		{
-			if (carriage_return)
-			{
-				TPUTS_TRACE("carriage_return");
-				tputs(carriage_return, 0, _nc_outch);
-			}
-			else
-				putchar('\r');
-			xold = 0;
-		}
+	    if (newline) {
+		TPUTS_TRACE("newline");
+		tputs(newline, 0, _nc_outch);
+	    } else
+		putchar('\n');
+	    l--;
+	    if (xold > 0) {
+		if (carriage_return) {
+		    TPUTS_TRACE("carriage_return");
+		    tputs(carriage_return, 0, _nc_outch);
+		} else
+		    putchar('\r');
+		xold = 0;
+	    }
 	}
     }
 
@@ -940,7 +904,7 @@ int mvcur(int yold, int xold, int ynew, int xnew)
 	ynew = screen_lines - 1;
 
     /* destination location is on screen now */
-    return(onscreen_mvcur(yold, xold, ynew, xnew, TRUE));
+    return (onscreen_mvcur(yold, xold, ynew, xnew, TRUE));
 }
 
 #if defined(TRACE) || defined(NCURSES_TEST)
@@ -961,35 +925,51 @@ const char *_nc_progname = "mvcur";
 
 static unsigned long xmits;
 
-int tputs(const char *string, int affcnt GCC_UNUSED, int (*outc)(int) GCC_UNUSED)
+/* these override lib_tputs.c */
+int
+tputs(const char *string, int affcnt GCC_UNUSED, int (*outc) (int) GCC_UNUSED)
 /* stub tputs() that dumps sequences in a visible form */
 {
     if (profiling)
 	xmits += strlen(string);
     else
 	(void) fputs(_nc_visbuf(string), stdout);
-    return(OK);
+    return (OK);
 }
 
-int putp(const char *string)
+int
+putp(const char *string)
 {
-    return(tputs(string, 1, _nc_outch));
+    return (tputs(string, 1, _nc_outch));
 }
 
-int _nc_outch(int ch)
+int
+_nc_outch(int ch)
 {
     putc(ch, stdout);
     return OK;
 }
 
-static char	tname[MAX_ALIAS];
+char PC = 0;			/* used by termcap library */
+speed_t ospeed = 0;		/* used by termcap library */
+int _nc_nulls_sent = 0;		/* used by 'tack' program */
 
-static void load_term(void)
+int
+delay_output(int ms GCC_UNUSED)
+{
+    return OK;
+}
+
+static char tname[MAX_ALIAS];
+
+static void
+load_term(void)
 {
     (void) setupterm(tname, STDOUT_FILENO, NULL);
 }
 
-static int roll(int n)
+static int
+roll(int n)
 {
     int i, j;
 
@@ -999,7 +979,8 @@ static int roll(int n)
     return (j % n);
 }
 
-int main(int argc GCC_UNUSED, char *argv[] GCC_UNUSED)
+int
+main(int argc GCC_UNUSED, char *argv[]GCC_UNUSED)
 {
     (void) strcpy(tname, termname());
     load_term();
@@ -1014,30 +995,32 @@ int main(int argc GCC_UNUSED, char *argv[] GCC_UNUSED)
     fputs("smcup:", stdout);
     putchar('\n');
 
-    for (;;)
-    {
-	int	fy, fx, ty, tx, n, i;
-	char	buf[BUFSIZ], capname[BUFSIZ];
+    for (;;) {
+	int fy, fx, ty, tx, n, i;
+	char buf[BUFSIZ], capname[BUFSIZ];
 
 	(void) fputs("> ", stdout);
 	(void) fgets(buf, sizeof(buf), stdin);
 
-	if (buf[0] == '?')
-	{
-(void) puts("?                -- display this help message");
-(void) puts("fy fx ty tx      -- (4 numbers) display (fy,fx)->(ty,tx) move");
-(void) puts("s[croll] n t b m -- display scrolling sequence");
-(void) printf("r[eload]         -- reload terminal info for %s\n", termname());
-(void) puts("l[oad] <term>    -- load terminal info for type <term>");
-(void) puts("d[elete] <cap>   -- delete named capability");
-(void) puts("i[nspect]        -- display terminal capabilities");
-(void) puts("c[ost]           -- dump cursor-optimization cost table");
-(void) puts("o[optimize]      -- toggle movement optimization");
-(void) puts("t[orture] <num>  -- torture-test with <num> random moves");
-(void) puts("q[uit]           -- quit the program");
-	}
-	else if (sscanf(buf, "%d %d %d %d", &fy, &fx, &ty, &tx) == 4)
-	{
+	if (buf[0] == '?') {
+	    (void) puts("?                -- display this help message");
+	    (void)
+		puts("fy fx ty tx      -- (4 numbers) display (fy,fx)->(ty,tx) move");
+	    (void) puts("s[croll] n t b m -- display scrolling sequence");
+	    (void)
+		printf("r[eload]         -- reload terminal info for %s\n",
+		termname());
+	    (void)
+		puts("l[oad] <term>    -- load terminal info for type <term>");
+	    (void) puts("d[elete] <cap>   -- delete named capability");
+	    (void) puts("i[nspect]        -- display terminal capabilities");
+	    (void)
+		puts("c[ost]           -- dump cursor-optimization cost table");
+	    (void) puts("o[optimize]      -- toggle movement optimization");
+	    (void)
+		puts("t[orture] <num>  -- torture-test with <num> random moves");
+	    (void) puts("q[uit]           -- quit the program");
+	} else if (sscanf(buf, "%d %d %d %d", &fy, &fx, &ty, &tx) == 4) {
 	    struct timeval before, after;
 
 	    putchar('"');
@@ -1047,10 +1030,9 @@ int main(int argc GCC_UNUSED, char *argv[] GCC_UNUSED)
 	    gettimeofday(&after, NULL);
 
 	    printf("\" (%ld msec)\n",
-		(long)(after.tv_usec - before.tv_usec + (after.tv_sec - before.tv_sec) * 1000000));
-	}
-	else if (sscanf(buf, "s %d %d %d %d", &fy, &fx, &ty, &tx) == 4)
-	{
+		(long) (after.tv_usec - before.tv_usec + (after.tv_sec -
+			before.tv_sec) * 1000000));
+	} else if (sscanf(buf, "s %d %d %d %d", &fy, &fx, &ty, &tx) == 4) {
 	    struct timeval before, after;
 
 	    putchar('"');
@@ -1060,66 +1042,53 @@ int main(int argc GCC_UNUSED, char *argv[] GCC_UNUSED)
 	    gettimeofday(&after, NULL);
 
 	    printf("\" (%ld msec)\n",
-		(long)(after.tv_usec - before.tv_usec + (after.tv_sec - before.tv_sec) * 1000000));
-	}
-	else if (buf[0] == 'r')
-	{
+		(long) (after.tv_usec - before.tv_usec + (after.tv_sec -
+			before.tv_sec) * 1000000));
+	} else if (buf[0] == 'r') {
 	    (void) strcpy(tname, termname());
 	    load_term();
-	}
-	else if (sscanf(buf, "l %s", tname) == 1)
-	{
+	} else if (sscanf(buf, "l %s", tname) == 1) {
 	    load_term();
-	}
-	else if (sscanf(buf, "d %s", capname) == 1)
-	{
-	    struct name_table_entry const	*np = _nc_find_entry(capname,
-							 _nc_info_hash_table);
+	} else if (sscanf(buf, "d %s", capname) == 1) {
+	    struct name_table_entry const *np = _nc_find_entry(capname,
+		_nc_info_hash_table);
 
 	    if (np == NULL)
 		(void) printf("No such capability as \"%s\"\n", capname);
-	    else
-	    {
-		switch(np->nte_type)
-		{
+	    else {
+		switch (np->nte_type) {
 		case BOOLEAN:
 		    cur_term->type.Booleans[np->nte_index] = FALSE;
-		    (void) printf("Boolean capability `%s' (%d) turned off.\n",
-				  np->nte_name, np->nte_index);
+		    (void)
+			printf("Boolean capability `%s' (%d) turned off.\n",
+			np->nte_name, np->nte_index);
 		    break;
 
 		case NUMBER:
-		    cur_term->type.Numbers[np->nte_index] = -1;
+		    cur_term->type.Numbers[np->nte_index] = ABSENT_NUMERIC;
 		    (void) printf("Number capability `%s' (%d) set to -1.\n",
-				  np->nte_name, np->nte_index);
+			np->nte_name, np->nte_index);
 		    break;
 
 		case STRING:
-		    cur_term->type.Strings[np->nte_index] = (char *)NULL;
+		    cur_term->type.Strings[np->nte_index] = ABSENT_STRING;
 		    (void) printf("String capability `%s' (%d) deleted.\n",
-				  np->nte_name, np->nte_index);
+			np->nte_name, np->nte_index);
 		    break;
 		}
 	    }
-	}
-	else if (buf[0] == 'i')
-	{
-	     dump_init((char *)NULL, F_TERMINFO, S_TERMINFO, 70, 0, FALSE);
-	     dump_entry(&cur_term->type, FALSE, TRUE, 0);
-	     putchar('\n');
-	}
-	else if (buf[0] == 'o')
-	{
-	     if (_nc_optimize_enable & OPTIMIZE_MVCUR)
-	     {
-		 _nc_optimize_enable &=~ OPTIMIZE_MVCUR;
-		 (void) puts("Optimization is now off.");
-	     }
-	     else
-	     {
-		 _nc_optimize_enable |= OPTIMIZE_MVCUR;
-		 (void) puts("Optimization is now on.");
-	     }
+	} else if (buf[0] == 'i') {
+	    dump_init((char *) NULL, F_TERMINFO, S_TERMINFO, 70, 0, FALSE);
+	    dump_entry(&cur_term->type, FALSE, TRUE, 0);
+	    putchar('\n');
+	} else if (buf[0] == 'o') {
+	    if (_nc_optimize_enable & OPTIMIZE_MVCUR) {
+		_nc_optimize_enable &= ~OPTIMIZE_MVCUR;
+		(void) puts("Optimization is now off.");
+	    } else {
+		_nc_optimize_enable |= OPTIMIZE_MVCUR;
+		(void) puts("Optimization is now on.");
+	    }
 	}
 	/*
 	 * You can use the `t' test to profile and tune the movement
@@ -1140,16 +1109,15 @@ int main(int argc GCC_UNUSED, char *argv[] GCC_UNUSED)
 	 * modes.  As long as the optimized times are less, the optimizer
 	 * is winning.
 	 */
-	else if (sscanf(buf, "t %d", &n) == 1)
-	{
+	else if (sscanf(buf, "t %d", &n) == 1) {
 	    float cumtime = 0, perchar;
-	    int speeds[] = {2400, 9600, 14400, 19200, 28800, 38400, 0};
+	    int speeds[] =
+	    {2400, 9600, 14400, 19200, 28800, 38400, 0};
 
-	    srand((unsigned)(getpid() + time((time_t *)0)));
+	    srand((unsigned) (getpid() + time((time_t *) 0)));
 	    profiling = TRUE;
 	    xmits = 0;
-	    for (i = 0; i < n; i++)
-	    {
+	    for (i = 0; i < n; i++) {
 		/*
 		 * This does a move test between two random locations,
 		 * Random moves probably short-change the optimizer,
@@ -1180,10 +1148,9 @@ int main(int argc GCC_UNUSED, char *argv[] GCC_UNUSED)
 	    perchar = cumtime / n;
 
 	    (void) printf("%d moves (%ld chars) in %d msec, %f msec each:\n",
-			  n, xmits, (int)cumtime, perchar);
+		n, xmits, (int) cumtime, perchar);
 
-	    for (i = 0; speeds[i]; i++)
-	    {
+	    for (i = 0; speeds[i]; i++) {
 		/*
 		 * Total estimated time for the moves, computation and
 		 * transmission both. Transmission time is an estimate
@@ -1198,12 +1165,11 @@ int main(int argc GCC_UNUSED, char *argv[] GCC_UNUSED)
 		 */
 		float overhead = speeds[i] * perchar / 1e6;
 
-		(void) printf("%6d bps: %3.2f char-xmits overhead; total estimated time %15.2f\n",
-			      speeds[i], overhead, totalest);
+		(void)
+		    printf("%6d bps: %3.2f char-xmits overhead; total estimated time %15.2f\n",
+		    speeds[i], overhead, totalest);
 	    }
-	}
-	else if (buf[0] == 'c')
-	{
+	} else if (buf[0] == 'c') {
 	    (void) printf("char padding: %d\n", SP->_char_padding);
 	    (void) printf("cr cost: %d\n", SP->_cr_cost);
 	    (void) printf("cup cost: %d\n", SP->_cup_cost);
@@ -1223,8 +1189,7 @@ int main(int argc GCC_UNUSED, char *argv[] GCC_UNUSED)
 	    (void) printf("cuu cost: %d\n", SP->_cuu_cost);
 	    (void) printf("hpa cost: %d\n", SP->_hpa_cost);
 	    (void) printf("vpa cost: %d\n", SP->_vpa_cost);
-	}
-	else if (buf[0] == 'x' || buf[0] == 'q')
+	} else if (buf[0] == 'x' || buf[0] == 'q')
 	    break;
 	else
 	    (void) puts("Invalid command.");
@@ -1234,7 +1199,7 @@ int main(int argc GCC_UNUSED, char *argv[] GCC_UNUSED)
     _nc_mvcur_wrap();
     putchar('\n');
 
-    return(0);
+    return (0);
 }
 
 #endif /* MAIN */
